@@ -11,6 +11,7 @@ use IOKit_sys::*;
 struct PowerMetricsResult {
     tasks: Vec<Task>,
     processor: ProcessorInfo,
+    smc: Smc,
 }
 
 #[derive(Debug, Deserialize)]
@@ -35,7 +36,7 @@ impl PowerMetrics {
         let o = Command::new("powermetrics")
             .args([
                 "--samplers",
-                "tasks,cpu_power",
+                "tasks,cpu_power,smc",
                 "--show-process-gpu",
                 "-n1",
                 "-i1000",
@@ -93,6 +94,10 @@ impl PowerMetrics {
                 .collect()
         }
     }
+
+    pub fn cpus_temperature(&self) -> Vec<f32> {
+        vec![self.last_result.smc.cpu_die]
+    }
 }
 
 #[derive(Debug, Default, Deserialize)]
@@ -125,6 +130,11 @@ struct Core {
     cpus: Vec<Cpu>,
 }
 
+#[derive(Debug, Default, Deserialize)]
+struct Smc {
+    cpu_die: f32,
+}
+
 pub struct IOKitRegistry {
     last_result: Vec<IOKitResult>,
 }
