@@ -5,14 +5,16 @@ use crate::platform::macos::PowerMetrics;
 #[cfg(target_os = "windows")]
 use crate::platform::windows::EtwTrace;
 #[cfg(target_os = "windows")]
-use crate::platform::windows::{Pdh, ProcessorInfo};
-use crate::Pid;
+use crate::platform::windows::{Pdh, ProcessorInfo, ThermalZoneInformation};
+use crate::{Error, Pid};
 use bitflags::bitflags;
-use sysinfo::{ProcessExt, RefreshKind, SystemExt};
+use std::fmt::{self, Display, Formatter};
+use sysinfo::{ProcessExt, SystemExt};
 
 #[derive(Default)]
 pub struct System {
     sysinfo_system: Option<sysinfo::System>,
+    refresh_kind: sysinfo::RefreshKind,
     #[cfg(target_os = "macos")]
     power_metrics: Option<PowerMetrics>,
     #[cfg(target_os = "macos")]
@@ -20,20 +22,23 @@ pub struct System {
     #[cfg(target_os = "windows")]
     pdh: Option<Pdh>,
     #[cfg(target_os = "windows")]
-    wmi_con: Option<wmi::WMIConnection>,
+    wmi_conn: Option<wmi::WMIConnection>,
     #[cfg(target_os = "windows")]
     etw_trace: Option<EtwTrace>,
 }
 
 impl System {
     #[allow(unused_variables)]
-    pub fn new<T: IntoIterator<Item = Pid>>(features: Features, pids: T) -> Self {
+    pub fn new<T: IntoIterator<Item = Pid>>(features: Features, pids: T) -> Result<Self, Error> {
         let mut system = System::default();
 
+        let mut use_sysinfo_system = false;
         if features.contains(Features::PROCESS) {
-            system.sysinfo_system = Some(sysinfo::System::new_with_specifics(
-                RefreshKind::new().with_processes(),
-            ));
+            system.refresh_kind = system.refresh_kind.with_processes();
+            use_sysinfo_system = true;
+        }
+        if use_sysinfo_system {
+            system.sysinfo_system = Some(sysinfo::System::new_with_specifics(system.refresh_kind));
         }
 
         if features.contains(Features::GPU) {
@@ -54,8 +59,7 @@ impl System {
             }
             #[cfg(target_os = "windows")]
             {
-                system.wmi_con =
-                    Some(wmi::WMIConnection::new(wmi::COMLibrary::new().unwrap().into()).unwrap());
+                system.wmi_conn = Some(wmi::WMIConnection::new(wmi::COMLibrary::new()?.into())?);
             }
         }
 
@@ -66,12 +70,28 @@ impl System {
             }
         }
 
-        system
+        if features.contains(Features::SMC) {
+            #[cfg(target_os = "macos")]
+            {
+                if system.power_metrics.is_none() {
+                    system.power_metrics = Some(PowerMetrics::new());
+                }
+            }
+            #[cfg(target_os = "windows")]
+            {
+                if system.wmi_conn.is_none() {
+                    system.wmi_conn =
+                        Some(wmi::WMIConnection::new(wmi::COMLibrary::new()?.into())?);
+                }
+            }
+        }
+
+        Ok(system)
     }
 
     pub fn update(&mut self) {
         if let Some(sysinfo_system) = &mut self.sysinfo_system {
-            sysinfo_system.refresh_processes();
+            sysinfo_system.refresh_specifics(self.refresh_kind);
         }
 
         #[cfg(target_os = "macos")]
@@ -110,19 +130,23 @@ impl System {
         Some(self.sysinfo_system.as_ref()?.process(pid)?.cmd())
     }
 
-    pub fn cpu_frequency(&self) -> Vec<f32> {
+    pub fn cpus_frequency(&self) -> Result<Vec<f32>, Error> {
         #[cfg(target_os = "macos")]
         {
-            self.power_metrics.as_ref().unwrap().cpu_frequency()
+            Ok(self
+                .power_metrics
+                .as_ref()
+                .ok_or(Error::FeatureMissing(Features::CPU_FREQUENCY))?
+                .cpu_frequency())
         }
         #[cfg(target_os = "windows")]
         {
-            let processor_info: Vec<ProcessorInfo> = self.wmi_con.as_ref().unwrap().raw_query("SELECT Name, PercentProcessorPerformance, ProcessorFrequency FROM Win32_PerfFormattedData_Counters_ProcessorInformation WHERE NOT Name LIKE '%_Total\'
-").unwrap();
-            processor_info
+            let processor_info: Vec<ProcessorInfo> = self.wmi_conn.as_ref().ok_or(Error::FeatureMissing(Features::CPU_FREQUENCY))?.raw_query("SELECT Name, PercentProcessorPerformance, ProcessorFrequency FROM Win32_PerfFormattedData_Counters_ProcessorInformation WHERE NOT Name LIKE '%_Total\'
+")?;
+            Ok(processor_info
                 .into_iter()
                 .map(|p| p.processor_frequency * p.percent_processor_performance / 100.0)
-                .collect()
+                .collect())
         }
     }
 
@@ -163,6 +187,31 @@ impl System {
             self.pdh.as_mut().unwrap().poll_gpu_usage(None)
         }
     }
+
+    pub fn cpus_temperature(&mut self) -> Result<Vec<f32>, Error> {
+        #[cfg(target_os = "macos")]
+        {
+            Ok(self
+                .power_metrics
+                .as_ref()
+                .ok_or(Error::FeatureMissing(Features::SMC))?
+                .cpus_temperature())
+        }
+        #[cfg(target_os = "windows")]
+        {
+            let wmi_conn = self
+                .wmi_conn
+                .as_ref()
+                .ok_or(Error::FeatureMissing(Features::SMC))?;
+            let thermal_zone_info: Vec<ThermalZoneInformation> = wmi_conn.raw_query(
+                "Select Temperature From Win32_PerfFormattedData_Counters_ThermalZoneInformation",
+            )?;
+            Ok(thermal_zone_info
+                .into_iter()
+                .map(|p| p.temperature - 273.15)
+                .collect())
+        }
+    }
 }
 
 bitflags! {
@@ -171,5 +220,56 @@ bitflags! {
         const GPU =             1 << 1;
         const CPU_FREQUENCY =   1 << 2;
         const FPS =             1 << 3;
+        const SMC =             1 << 4;
+    }
+}
+
+impl Display for Features {
+    #[allow(unused_assignments)]
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        let mut first = true;
+
+        if self.contains(Features::PROCESS) {
+            if !first {
+                write!(f, "|")?;
+            } else {
+                first = false;
+            }
+            write!(f, "{}", "PROCESS")?;
+        }
+        if self.contains(Features::GPU) {
+            if !first {
+                write!(f, "|")?;
+            } else {
+                first = false;
+            }
+            write!(f, "{}", "GPU")?;
+        }
+        if self.contains(Features::CPU_FREQUENCY) {
+            if !first {
+                write!(f, "|")?;
+            } else {
+                first = false;
+            }
+            write!(f, "{}", "CPU_FREQUENCY")?;
+        }
+        if self.contains(Features::FPS) {
+            if !first {
+                write!(f, "|")?;
+            } else {
+                first = false;
+            }
+            write!(f, "{}", "FPS")?;
+        }
+        if self.contains(Features::SMC) {
+            if !first {
+                write!(f, "|")?;
+            } else {
+                first = false;
+            }
+            write!(f, "{}", "SMC")?;
+        }
+
+        Ok(())
     }
 }
